#ifndef PTCHAN_HH
#define PTCHAN_HH

#include "TSeries.hh"
#include "FixedLenTS.hh"
#include "DecimateBy2.hh"

#include <string>
#include <list>
#include <vector>
#include <iosfwd>

//---------------------------------------Include C headers

#include "detect.h"
#include "ligoconfig.h"
#include "ligocont.h"
#include "ligostd.h"

extern "C" {
void Cont_setup(con_sto *);
int *E_buffer(int, int);
unsigned char E_create(struct detect_info *, int, int, const char *, 
          int *, unsigned char (*)(), double *);
  unsigned char E_detect(struct detect_info*, struct s_onset*, int*, const int);
unsigned char E_filter(struct detect_info*);
unsigned char Event();
unsigned char FfirBB();
void Onset(int,int,int,int,int,int,int,int,int,int,
           struct detect_info*, struct s_onset*, int*, const int);
void Onsetq(struct detect_info*, struct s_onset*, int*, const int);
unsigned char P_one(int);
}

class Dacc;

/*  PTChan holds the channel time series. Each 
 *  time the channel data are read in the PTChan class gets a pointer to 
 *  the time series and converts the data to floating point.  Pipes are 
 *  set up for filtering data.
 */
class PTChan {
public:
  PTChan(void);
  explicit PTChan(const char* name);
  PTChan(const PTChan& x);
  ~PTChan(void);
  
  const char* getChannel(void) const;     // Get name of channel
  void setSps(long int samSec);
  void setup(Dacc& In);
  const TSeries& refTSeries(void) const;
  void setDecimation(int, int);           // Set decimation rate
  TSeries apply(const TSeries& in);
  void reset(void);
  void setEnableCond(const std::string& cond);
  bool findit(const int, s_onset*);

//--------------------------------------- Inline functions
  void setDebug(int level);           // Set up debugging
  int getDebug(void) const;                                        
  size_t getData( size_t len, int* data) const; //Extract data as integers
  const TSeries& refData(void) const; 
  size_t getNSample(void) const;
  const char* getEnableCond(void) const;
  const char* getName(void) const;        // Get name of monitor
  double getAvgRate(void) const;
  void setSettle(Interval dt);
  void setThreshold(float repThresh);
  bool getTrigger(void) const;
  void setTrigger(bool trig);
  const s_onset& refOnsetData(void) const;

//------------------------------------- C structures
  struct detect_info detect;
  struct s_onset onset;
  struct r_time startt;

  s_onset* oPtr;

//--------------------------------------public variables
  long int mSps;
  float mThresh;
  int mOnsetN;
  long int mNTotal;
  Time mtStart;

private:
  std::string mName;
  std::string mChannel;
  std::string mEnableCond;              // Enabling condition name

  bool mTrigger;
  Dacc*  mDacc;
  TSeries* mTS;
  Time mLast;
  DecimateBy2 mDec;

  TSeries  mRawData;
  double *coefArray;
  int * mDataArray;
  double * mCoefPtr;

  Interval mSettle;
  Interval mObserved;
  int maxDataPoints;
  int maxLookBack;
  int maxCoef;
  int mDebug;
  int nOnset;
  int counter;
 
};

//======================================= PTChan inline functions

inline void
PTChan::setDebug(int level) {
  mDebug = level;
}

inline int
PTChan::getDebug(void) const {
  return mDebug;
}

inline size_t
PTChan::getData(size_t len, int* data) const {
  size_t mNWords;
  return mNWords;
}

inline const TSeries&
PTChan::refData(void) const {
  return mRawData;
}

inline size_t
PTChan::getNSample(void) const {
  size_t N;
  return N;
}

inline const char*
PTChan::getEnableCond(void) const {
  return mEnableCond.c_str();
}

inline const char*
PTChan::getName(void) const {
  return mName.c_str();
}

inline void
PTChan::setSettle(Interval dt){
  mSettle = dt;
}

inline void
PTChan::setThreshold(float repThresh){
  mThresh = repThresh;
}

inline double
PTChan::getAvgRate(void) const {
  if (double(mObserved) == 0.0) return 0.0;
  return double(mNTotal)/double(mObserved); 
}

inline bool
PTChan::getTrigger(void) const {
  return mTrigger;
}

inline void
PTChan::setTrigger(bool trig) {
  mTrigger = trig;
}

inline const s_onset&
PTChan::refOnsetData(void) const {
return *oPtr;
}

#endif  // !defined(PTCHAN_HH)
