(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     25324,        673]*)
(*NotebookOutlinePosition[     26004,        696]*)
(*  CellTagsIndexPosition[     25960,        692]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["NDS Interface",
  FontSlant->"Italic"]], "Subtitle",
  PageWidth->WindowWidth],

Cell[CellGroupData[{

Cell["Header", "Section",
  PageWidth->WindowWidth],

Cell[BoxData[
    \(\(\( (*\(:\)\(Name : \ 
          LIGO`nds`\)\ *) \)\(\n\)\(\n\)\( (*\(\(:\)\(Author : \ 
          Daniel\ Sigg\)\), \ 
      December\ 1999\ *) \)\(\n\)\(\n\)\( (*\(:\)\(Summary : \ 
          This\ package\ provides\ access\ to\ LIGO\ data\ through\ a\ \
MathLink\ interface\ to\ the\ Network\ data\ \(\(server\)\(.\)\)\)\ *) \)\(\n\
\)\(\n\)\( (*\(\(:\)\(Keywords : \ Network\ data\ server\)\), \ 
      LIGO\ *) \)\(\n\)\(\n\)\( (*\(:\)\(\(Sources\)\(:\)\)\ *) \)\(\n\)\(\n\
\)\(BeginPackage\ ["\<LIGO`nds`\>"];\)\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Usage message", "Section",
  PageWidth->WindowWidth],

Cell[CellGroupData[{

Cell["Options", "Subsection",
  PageWidth->WindowWidth],

Cell[BoxData[{
    \(\(LHO::usage = "\<The option value LHO is used to represente the site \
of the LIGO Hanford Observatory.\>";\)\), "\n", 
    \(\(LLO::usage = "\<The option value LHO is used to represente the site \
of the LIGO Hanford Observatory.\>";\)\), "\n", 
    \(\(Site::usage = "\<Site is an option which is used to specify the \
loaction of the data server. Supported values are LHO, LLO or a string \
describing the port number and server name in the format \
\"1234@daniel.ligo-wa.caltech.edu\"\>";\)\), "\n", 
    \(\($DefaultSite::usage = "\<$DefaultSite is used to specify the default \
location of the NDS data server.\>";\)\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[{
    \(\(ColumnFormat::usage = "\<ColumnFormat is an option representing a \
data format where each channel is stored as an individual data \
array.\>";\)\), "\n", 
    \(\(RowFormat::usage = "\<RowFormat is an option representing a data \
format where the data is stored in n-tuples of channel points.\>";\)\), "\n", \

    \(\(DataFormat::usage = "\<The option DataFormat specifies the format of \
the returned data array. Possible values are RowFormat or ColumnFormat. In \
row format the data is returned as a list of n-tuples representing the time, \
the value of the first channel, the value of the second channel, etc. In \
column format the data is returned as a list of channels data arrays. The \
first data array always describes the time.\>";\)\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[{
    \(\(SimpleFormat::usage = "\<The option SimpleFormat specifies the format \
of the returned trend data array. Possibe values are True or False. A simple \
return array consists of just the time and mean trend value, whereas the full \
return array consists of time, counts, mean value, stdandard deviation, \
minimum and maximum.\>";\)\n\), "\n", 
    \(\(DataRate::usage = "\<DataRate is an option to specify the sampling \
rate of the returned data array. The value is specified in units of Hz.\>";\)\
\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(\(Heterodyne::usage = "\<The Heterodyne option is used to down-convert \
the data by the specified rate before it is returned. The value is specified \
in units of Hz. The returned data values will be complex.\>";\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[{
    StyleBox[\(ChannelName::usage = "\<ChannelName is an options used by \
NDSGetChannelInfo to return the channel name.\>";\),
      FontFamily->"Courier",
      FontWeight->
        "Bold"], "\n", \(InterferometerId::usage = "\<InterferometerId is an \
options used by NDSGetChannelInfo to return the interferometer identification \
number.\>";\), "\n", \(DCU::usage = "\<DCU is an options used by \
NDSGetChannelInfo to return the DCU information.\>";\), "\n", \
\(NDSDataType::usage = "\<NDSDataType is an options used by NDSGetChannelInfo \
to return the native data type.\>";\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(\(NDSClose::usage = "\<NDSClose[] terminates the connection to the NDS \
server.\>";\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(\(NDSGetChannel::usage = "\<NDSGetChannel[] obtains a list of all \
currently available channel names. The following option can be given: Site \
(default $DefaultSite) which specifies the location of the NDS \
server.\>";\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(\(NDSGetChannelInfo::usage = "\<NDSGetChannelInfo[] obtains a list of \
all currently available channel names and the information assciated with \
them. Each channel is represented by a list with the following elements: \
{ChannelName->\"name\", DataRate\[Rule]rate,InterferometerId\[Rule]ifo, DCU\
\[Rule]id, NDSDataType\[Rule]dtype}. The data rate is specified in units of \
Hz; the interferometer number is either 0 for the 4k intereformeter or 1 for \
the 2k; the DCU id is the idendification number of data collection unit which \
collected the channel; and the data type is one of the following: 1 \[Dash] \
short integer (16 bit), 2 \[Dash] integer (32 bit), 3 \[Dash] long integer \
(64 bit), 4 \[Dash] single precision floating point number (32 bit), and 5 \
\[Dash] double precision floating point number (64 bit). The following option \
can be given: Site (default $DefauiltSite)which specifies the location of the \
NDS server.\>";\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(\(NDSGetData::usage = "\<NDSGetData[\"channel\"], start, duratio] \
obtains data for a single channel from the network data server. \
NDSGetData[\"chn1\", \"chn2\", ...}, start, duration] obtains data for \
multiple channels from the network datya server. The following options can be \
given: DataRate (default 256) which specifies the sampling rate of the \
returned data, DataFormat (default ColumnFormat) which specifies the format \
of the returned data array, Heterodyne (default 0) which specifies the \
down-conversion frequency, and Site (default $DefaultSite) which specifies \
the location of the NDS server.\>";\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(\(NDSGetTrend::usage = "\<NDSGetTrend[\"channel\", start, duratio] \
obtains trend data for a single channel from the network data server. \
NDSGetTrend[\"chn1\", \"chn2\", ...}, start, duration] obtains trend data for \
multiple channels from the network datya server. Each returned data point \
constist of an n-tuple consisting of the time, the mean value, the rms value, \
the maximum and the minimum. The returned data format is always in row \
format. The data rate has to be smaller or equal one Hertz. For rates faster \
than once a minute, the rate must be an integer fraction of one. For rate \
slower than once a minute the rate must be an integer fraction of 1/60. The \
following options can be given: DataRate (default 1)which specifies the \
sampling rate of the returned data, and Site (default $DefaultSite) which \
specifies the location of the NDS server.\>";\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(NDSTrendEliminateInvalid::usage = "\<Eliminates invalid data points \
from a returned trend data array. Invalid data points are recognized by an N \
of zero.\>"\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Error messages", "Subsection",
  PageWidth->WindowWidth],

Cell[BoxData[{
    \(\(nds::nosite = "\<Unsupported site name.\>";\)\), "\n", 
    \(\(nds::optx = "\<Unsupported option(s) `1` encountered.\>";\)\), "\n", 
    \(\(nds::nochn = "\<No channel names.\>";\)\), "\n", 
    \(\(nds::time = "\<Invalid time specification.\>";\)\), "\n", 
    \(\(nds::rate = "\<Invalid rate specification (must be a power of \
two).\>";\)\), "\n", 
    \(\(nds::fdown = "\<Invalid heterodyn frequency.\>";\)\), "\n", 
    \(\(nds::duration = "\<Invalid time duration.\>";\)\), "\n", 
    \(\(nds::format = "\<Illegal trend format.\>";\)\), "\n", 
    \(\(nds::server = "\<NDS server not available.\>";\)\), "\n", 
    \(\(nds::server2 = "\<Unable to request data from NDS server.\>";\)\), "\n\
", 
    \(\(nds::server3 = "\<Server transmission error.\>";\)\), "\n", 
    \(\(nds::chn = "\<Invalid channel names specified.\>";\)\), "\n", 
    \(\(nds::nomem = "\<Unsufficient memory.\>";\)\), "\n", 
    \(\(nds::pre = "\<Unable to initialize preprocessor.\>";\)\), "\n", 
    \(\(nds::unknown = "\<Unexpected error.\>";\)\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Protected", "Subsection",
  PageWidth->WindowWidth],

Cell[BoxData[
    RowBox[{\(Begin["\<`Private`\>"]\), ";", "\n", 
      "\n", \(Attributes[LHO] = {Protected}\), ";", 
      "\n", \(Attributes[LLO] = {Protected}\), ";", 
      "\n", \(Attributes[Site] = {Protected}\), ";", "\n", 
      "\n", \(Attributes[ColumnFormat] = {Protected}\), ";", 
      "\n", \(Attributes[RowFormat] = {Protected}\), ";", 
      "\n", \(Attributes[DataFormat] = {Protected}\), ";", "\n", 
      "\n", \(Attributes[DataRate] = {Protected}\), ";", "\n", 
      RowBox[{
        RowBox[{"Attributes", "[", 
          StyleBox["Heterodyne",
            FontFamily->"Courier",
            FontWeight->"Bold"], "]"}], "=", \({Protected}\)}], ";", "\n", 
      "\n", 
      RowBox[{
        RowBox[{"Attributes", "[", 
          StyleBox["ChannelName",
            FontFamily->"Courier",
            FontSize->11,
            FontWeight->"Bold"], "]"}], "=", \({Protected}\)}], ";", 
      "\n", \(Attributes[InterferometerId] = {Protected}\), ";", 
      "\n", \(Attributes[DCU] = {Protected}\), ";", 
      "\n", \(Attributes[NDSDataType] = {Protected}\), ";"}]], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Globals", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(NDSLocations = \n\t{LHO \[Rule] "\<9090@stone.ligo-wa.caltech.edu\>", \
\n\t\tLLO \[Rule] "\<9090@delaronde.ligo-la.caltech.edu\>"};\)\), "\n", 
    \(\(CurrentLocation = "\<\>";\)\), "\n", 
    \(\(NDSLink = 0;\)\), "\n", 
    \(\($DefaultSite = LHO;\)\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Utilities", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Check options", "Subsection",
  PageWidth->WindowWidth],

Cell[BoxData[
    \(IllegaleOptions[\ command_Symbol, \ opts___\ ]\  := \n\t
      Select[\ Flatten[{opts}], \(! MemberQ[First\  /@ \ Options[command], \ 
              First[#]]\) &\ ]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(CheckOptions[command_Symbol, \ options___] := 
      Module[{l}, \ l = IllegaleOptions[command, options]; 
        If[Length[l] > 0, Message[nds::optx, \ l]; Abort[], 0]]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Concatinate string list", "Subsection",
  PageWidth->WindowWidth],

Cell[BoxData[
    \(ChannelArg[channel_List /; VectorQ[channel, Head[#] === String &]] := 
      StringJoin[\(# <> "\<\t\>" &\) /@ channel]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Establish a link", "Subsection",
  PageWidth->WindowWidth],

Cell[BoxData[{
    \(\(\(EstablishLink[location_String] := \n\t
      Module[{link}, \n\t\tIf[location === CurrentLocation, 
          NDSLink, \n\t\t\tNDSClose[]; \n\t\t\tlink = 
            Install[LinkConnect[location, 
                LinkProtocol -> "\<TCP\>"]]; \n\t\t\tIf[
            Head[link] =!= LinkObject, Abort[]]; \n\t\t\tCurrentLocation = \ 
            location; \n\t\t\tIf[Head[NDSLink] == LinkObject, 
            LinkClose[NDSLink]]; \n\t\tNDSLink = link]]\)\(\n\)
    \)\), "\n", 
    \(EstablishLink[location_] := \n\t
      Module[{loc}, \n\t\tloc\  = \ location /. NDSLocations; \n\t\tIf[
          Head[loc] === String, 
          EstablishLink[loc], \n\t\t\t\tMessage[nds::nosite]; 
          Abort[]]]\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convert real to complex", "Subsection",
  PageWidth->WindowWidth],

Cell[BoxData[
    \(Real2Complex[l_List /; VectorQ[l, NumberQ]] := \(Complex @@ # &\) /@ 
        Partition[l, 2]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["CheckForErrors", "Subsection",
  PageWidth->WindowWidth],

Cell[BoxData[
    \(CheckForErrors1[l_] := \n\t
      If[Head[l] === Integer, \n\t\tSwitch[l, \(-1\), 
          Message[nds::nochn], \n\t\t\(-2\), \ 
          Message[nds::time], \n\t\t\(-3\), \ 
          Message[nds::rate], \n\t\t\(-4\), \ 
          Message[nds::fdown], \n\t\t\(-5\), \ 
          Message[nds::duration], \n\t\t\(-6\), \ 
          Message[nds::server], \n\t\t\(-7\), \ 
          Message[nds::chn], \n\t\t\(-8\), \ 
          Message[nds::nochn], \n\t\t\(-9\), \ 
          Message[nds::rate], \n\t\t\(-10\), \ 
          Message[nds::nomem], \n\t\t\(-11\), \ 
          Message[nds::pre], \n\t\t\(-12\), \ 
          Message[nds::server2], \n\t\t\(-13\), \ 
          Message[nds::server3], \n\t\t\(-99\), Null, \n\t\t_, \ 
          Message[nds::unknown]\n\t\t]; Abort[]]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(CheckForErrors2[l_] := \n\t
      If[Head[l] === Integer, \n\t\tSwitch[l, \(-1\), 
          Message[nds::nochn], \n\t\t\(-2\), \ 
          Message[nds::time], \n\t\t\(-3\), \ 
          Message[nds::rate], \n\t\t\(-4\), \ 
          Message[nds::duration], \n\t\t\(-5\), \ 
          Message[nds::server], \n\t\t\(-6\), \ 
          Message[nds::chn], \n\t\t\(-7\), \ 
          Message[nds::nochn], \n\t\t\(-8\), \ 
          Message[nds::nomem], \n\t\t\(-9\), \ 
          Message[nds::server2], \n\t\t\(-10\), \ 
          Message[nds::format], \n\t\t\(-99\), Null, \n\t\t_, \ 
          Message[nds::unknown]\n\t\t]; Abort[], l]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSClose", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(Options[NDSClose] = {};\)\), "\n", 
    \(NDSClose[] := 
      Module[{}, \n\t\tIf[Head[NDSLink] === LinkObject, 
          LinkClose[NDSLink]]; \n\t\t\tNDSLink = 0; 
        CurrentLocation = \ "\<\>";]\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSGetChannel", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(Options[
          NDSGetChannel] = {Site \[RuleDelayed] $DefaultSite};\)\), "\n", 
    \(NDSGetChannel[opts___] := \n\t
      Module[{site}, \n\t\tCheckOptions[NDSGetChannel, \ opts]; 
        site = \(Site /. {opts}\) /. Options[NDSGetChannel]; 
        EstablishLink[site]; Global`ndsgetchannels[]]\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSGetChannelInfo", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(Options[
          NDSGetChannelInfo] = {Site \[RuleDelayed] $DefaultSite};\)\), "\n", \

    \(NDSGetChannelInfo[opts___] := \n\t
      Module[{site, \ info}, \n\t\tCheckOptions[NDSGetChannelInfo, \ opts]; 
        site = \(Site /. {opts}\) /. Options[NDSGetChannelInfo]; 
        EstablishLink[site]; \n\t\tinfo = 
          Global`ndsgetchannelinfo[]; \n\t\({ChannelName \[Rule] #\
\[LeftDoubleBracket]1\[RightDoubleBracket], 
              DataRate \[Rule] #\[LeftDoubleBracket]2\[RightDoubleBracket], 
              InterferometerId \[Rule] #\[LeftDoubleBracket]3\
\[RightDoubleBracket], 
              DCU \[Rule] #\[LeftDoubleBracket]4\[RightDoubleBracket], 
              NDSDataType \[Rule] \
#\[LeftDoubleBracket]5\[RightDoubleBracket]} &\) /@ info]\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSGetData", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(Options[NDSGetData] = {Site \[RuleDelayed] $DefaultSite, 
          DataRate \[Rule] 256, Heterodyne \[Rule] 0, 
          DataFormat \[Rule] ColumnFormat};\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(NDSGetData[channel_String, 
        start_List /; 
          VectorQ[start, NumberQ] \[And] \((Length[start] === 2)\), 
        duration_?NumberQ, opts___] := \n\t
      NDSGetData[{channel}, start, duration, opts]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(NDSGetData[channel_String, 
        start_List /; 
          VectorQ[start, NumberQ] \[And] \((Length[start] === 6)\), 
        duration_?NumberQ, opts___] := \n\t
      NDSGetData[{channel}, start, duration, opts]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(NDSGetData[
        channel_List /; 
          VectorQ[channel, Head[#] === String &], {gpssec_?NumberQ, 
          gpsnsec_?NumberQ}, duration_?NumberQ, opts___] := \n\t
      Module[{site, drate, format, fdown, chns, data}, 
        CheckOptions[NDSGetData, \ opts]; 
        site = \(Site /. {opts}\) /. Options[NDSGetData]; 
        drate = \(DataRate /. {opts}\) /. Options[NDSGetData]; 
        format = \(DataFormat /. {opts}\) /. Options[NDSGetData]; 
        fdown = \(Heterodyne /. {opts}\) /. Options[NDSGetData]; 
        EstablishLink[site]; \n\t\tchns = ChannelArg[channel]; \n\t\tdata = 
          Global`ndsgetdatagps[chns, drate, fdown, gpssec, gpsnsec, 
            duration]; \n\t\tCheckForErrors1[data]; \n\t\tIf[fdown > 0, 
          data = Real2Complex /@ data]; \n\t\tIf[format === RowFormat, 
          Transpose[data], data]\n\t\t]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(NDSGetData[
        channel_List /; 
          VectorQ[channel, Head[#] === String &], {year_Integer, 
          month_Integer, day_Integer, hour_Integer, min_Integer, 
          sec_?NumberQ}, duration_?NumberQ, opts___] := \n\t
      Module[{site, drate, format, fdown, chns, data}, 
        CheckOptions[NDSGetData, \ opts]; 
        site = \(Site /. {opts}\) /. Options[NDSGetData]; 
        drate = \(DataRate /. {opts}\) /. Options[NDSGetData]; 
        format = \(DataFormat /. {opts}\) /. Options[NDSGetData]; 
        fdown = \(Heterodyne /. {opts}\) /. Options[NDSGetData]; 
        EstablishLink[site]; \n\t\tchns = ChannelArg[channel]; \n\t\tdata = 
          Global`ndsgetdatautc[chns, drate, fdown, year, month, day, hour, 
            min, sec, duration]; \n\t\tIf[CheckForErrors1[data], 
          Return[Abort[]]]; \n\t\tIf[fdown > 0, 
          data = Real2Complex /@ data]; \n\t\tIf[format === RowFormat, 
          Transpose[data], data]\n\t\t]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSGetTrend", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(Options[NDSGetTrend] = {Site \[RuleDelayed] $DefaultSite, 
          DataRate \[Rule] 256, SimpleFormat \[Rule] True};\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(NDSGetTrend[channel_String, 
        start_List /; 
          VectorQ[start, NumberQ] \[And] \((Length[start] === 2)\), 
        duration_?NumberQ, opts___] := \n\t
      NDSGetTrend[{channel}, start, duration, opts]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(NDSGetTrend[channel_String, 
        start_List /; 
          VectorQ[start, NumberQ] \[And] \((Length[start] === 6)\), 
        duration_?NumberQ, opts___] := \n\t
      NDSGetTrend[{channel}, start, duration, opts]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(NDSGetTrend[
        channel_List /; 
          VectorQ[channel, Head[#] === String &], {gpssec_?NumberQ, 
          gpsnsec_?NumberQ}, duration_?NumberQ, opts___] := \n\t
      Module[{site, drate, format, fstr, chns}, 
        CheckOptions[NDSGetTrend, \ opts]; 
        site = \(Site /. {opts}\) /. Options[NDSGetTrend]; 
        drate = \(DataRate /. {opts}\) /. 
            Options[NDSGetTrend]; \[IndentingNewLine]format = \(SimpleFormat /. \
{opts}\) /. Options[NDSGetTrend]; \[IndentingNewLine]fstr = 
          If[format === 
              True, "\<y\>", "\<nysmM\>"]; \[IndentingNewLine]EstablishLink[
          site]; \[IndentingNewLine]chns = 
          ChannelArg[
            channel]; \
\[IndentingNewLine]CheckForErrors2[\[IndentingNewLine]Global`ndsgettrendgps[
            chns, drate, gpssec, gpsnsec, duration, 
            fstr]]\t\t\n\t\t]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True],

Cell[BoxData[
    \(NDSGetTrend[
        channel_List /; 
          VectorQ[channel, Head[#] === String &], {year_Integer, 
          month_Integer, day_Integer, hour_Integer, min_Integer, 
          sec_?NumberQ}, duration_?NumberQ, opts___] := \n\t
      Module[{site, drate, format, fstr, chns}, 
        CheckOptions[NDSGetTrend, \ opts]; 
        site = \(Site /. {opts}\) /. Options[NDSGetTrend]; 
        drate = \(DataRate /. {opts}\) /. 
            Options[NDSGetTrend]; \[IndentingNewLine]format = \(SimpleFormat /. \
{opts}\) /. Options[NDSGetTrend]; \[IndentingNewLine]fstr = 
          If[format === 
              True, "\<y\>", "\<nysmM\>"]; \[IndentingNewLine]EstablishLink[
          site]; \[IndentingNewLine]chns = 
          ChannelArg[
            channel]; \
\[IndentingNewLine]CheckForErrors2[\[IndentingNewLine]Global`ndsgettrendutc[
            chns, drate, year, month, day, hour, min, sec, duration, 
            fstr]]\n\t\t]\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSTrendEliminateInvalid", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(Options[NDSTrendEliminateInvalid] = {};\)\), "\n", 
    \(NDSTrendEliminateInvalid[
        trend_ /; \((And @@ \((\((MatrixQ[#, NumberQ] &)\) /@ 
                  trend)\))\)]\  := \n\t\(Select[#, \((#\[LeftDoubleBracket]2\
\[RightDoubleBracket] > 0.9)\) &] &\) /@ trend\)}], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Epilog", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\($Epilog = NDSClose;\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["End", "Section",
  PageWidth->WindowWidth,
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(\(End\ []\)\(;\)\(\ \)\( (*\ 
      LIGO`nds`Private`\ *) \)\(\n\)\(\n\)\(EndPackage\ []\)\(;\)\(\ \)\( (*\ 
      LIGO`nds`\ *) \)\)\)], "Input",
  PageWidth->WindowWidth,
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.2 for Microsoft Windows",
ScreenRectangle->{{0, 1920}, {0, 1113}},
AutoGeneratedPackage->Automatic,
WindowSize->{738, 946},
WindowMargins->{{235, Automatic}, {Automatic, 11}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 102, 2, 52, "Subtitle"],

Cell[CellGroupData[{
Cell[1903, 59, 51, 1, 73, "Section"],
Cell[1957, 62, 607, 11, 250, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[2601, 78, 58, 1, 73, "Section"],

Cell[CellGroupData[{
Cell[2684, 83, 55, 1, 38, "Subsection"],
Cell[2742, 86, 712, 12, 250, "Input",
  InitializationCell->True],
Cell[3457, 100, 838, 14, 270, "Input",
  InitializationCell->True],
Cell[4298, 116, 591, 10, 190, "Input",
  InitializationCell->True],
Cell[4892, 128, 302, 5, 90, "Input",
  InitializationCell->True],
Cell[5197, 135, 664, 12, 210, "Input",
  InitializationCell->True],
Cell[5864, 149, 173, 4, 30, "Input",
  InitializationCell->True],
Cell[6040, 155, 310, 6, 90, "Input",
  InitializationCell->True],
Cell[6353, 163, 1032, 15, 270, "Input",
  InitializationCell->True],
Cell[7388, 180, 712, 11, 210, "Input",
  InitializationCell->True],
Cell[8103, 193, 967, 14, 270, "Input",
  InitializationCell->True],
Cell[9073, 209, 247, 5, 70, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[9357, 219, 62, 1, 38, "Subsection"],
Cell[9422, 222, 1116, 19, 310, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[10575, 246, 57, 1, 38, "Subsection"],
Cell[10635, 249, 1151, 25, 350, "Input",
  InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11835, 280, 100, 3, 73, "Section",
  Evaluatable->False],
Cell[11938, 285, 344, 7, 130, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[12319, 297, 102, 3, 73, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[12446, 304, 61, 1, 38, "Subsection"],
Cell[12510, 307, 252, 5, 50, "Input",
  InitializationCell->True],
Cell[12765, 314, 257, 5, 70, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[13059, 324, 71, 1, 38, "Subsection"],
Cell[13133, 327, 206, 4, 50, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[13376, 336, 64, 1, 38, "Subsection"],
Cell[13443, 339, 796, 16, 310, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[14276, 360, 71, 1, 38, "Subsection"],
Cell[14350, 363, 180, 4, 30, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[14567, 372, 62, 1, 38, "Subsection"],
Cell[14632, 375, 863, 18, 370, "Input",
  InitializationCell->True],
Cell[15498, 395, 721, 15, 310, "Input",
  InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[16268, 416, 101, 3, 73, "Section",
  Evaluatable->False],
Cell[16372, 421, 294, 7, 90, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[16703, 433, 106, 3, 73, "Section",
  Evaluatable->False],
Cell[16812, 438, 392, 8, 130, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[17241, 451, 110, 3, 73, "Section",
  Evaluatable->False],
Cell[17354, 456, 850, 17, 170, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[18241, 478, 103, 3, 73, "Section",
  Evaluatable->False],
Cell[18347, 483, 247, 5, 50, "Input",
  InitializationCell->True],
Cell[18597, 490, 301, 7, 70, "Input",
  InitializationCell->True],
Cell[18901, 499, 301, 7, 70, "Input",
  InitializationCell->True],
Cell[19205, 508, 943, 17, 270, "Input",
  InitializationCell->True],
Cell[20151, 527, 1051, 19, 310, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[21239, 551, 104, 3, 73, "Section",
  Evaluatable->False],
Cell[21346, 556, 209, 4, 30, "Input",
  InitializationCell->True],
Cell[21558, 562, 303, 7, 70, "Input",
  InitializationCell->True],
Cell[21864, 571, 303, 7, 70, "Input",
  InitializationCell->True],
Cell[22170, 580, 949, 20, 250, "Input",
  InitializationCell->True],
Cell[23122, 602, 1021, 21, 270, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[24180, 628, 117, 3, 73, "Section",
  Evaluatable->False],
Cell[24300, 633, 363, 7, 70, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[24700, 645, 99, 3, 73, "Section",
  Evaluatable->False],
Cell[24802, 650, 110, 3, 30, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[24949, 658, 96, 3, 73, "Section",
  Evaluatable->False],
Cell[25048, 663, 248, 6, 70, "Input",
  InitializationCell->True]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

